package com.sun.xml.internal.ws.api.message.saaj;

import javax.xml.soap.AttachmentPart;
import com.sun.xml.internal.ws.api.message.AttachmentEx;
import com.sun.xml.internal.ws.api.message.Attachment;
import org.xml.sax.SAXException;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import com.sun.xml.internal.bind.marshaller.SAX2DOMEx;
import javax.xml.stream.XMLStreamException;
import com.sun.xml.internal.ws.message.saaj.SAAJMessage;
import com.sun.xml.internal.ws.api.message.Packet;
import com.sun.xml.internal.ws.api.SOAPVersion;
import com.sun.xml.internal.ws.api.message.Message;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPException;
import java.util.Iterator;
import com.sun.xml.internal.ws.util.ServiceFinder;
import javax.xml.soap.MessageFactory;

public class SAAJFactory
{
    private static final SAAJFactory instance;
    
    public SAAJFactory() {
    }
    
    public static MessageFactory getMessageFactory(String protocol) throws SOAPException {
        for (SAAJFactory s : ServiceFinder.find(SAAJFactory.class)) {
            MessageFactory mf = s.createMessageFactory(protocol);
            if (mf != null) {
                return mf;
            }
            continue;
        }
        return SAAJFactory.instance.createMessageFactory(protocol);
    }
    
    public static SOAPFactory getSOAPFactory(String protocol) throws SOAPException {
        for (SAAJFactory s : ServiceFinder.find(SAAJFactory.class)) {
            SOAPFactory sf = s.createSOAPFactory(protocol);
            if (sf != null) {
                return sf;
            }
            continue;
        }
        return SAAJFactory.instance.createSOAPFactory(protocol);
    }
    
    public static Message create(SOAPMessage saaj) {
        for (SAAJFactory s : ServiceFinder.find(SAAJFactory.class)) {
            Message m = s.createMessage(saaj);
            if (m != null) {
                return m;
            }
            continue;
        }
        return SAAJFactory.instance.createMessage(saaj);
    }
    
    public static SOAPMessage read(SOAPVersion soapVersion, Message message) throws SOAPException {
        for (SAAJFactory s : ServiceFinder.find(SAAJFactory.class)) {
            SOAPMessage msg = s.readAsSOAPMessage(soapVersion, message);
            if (msg != null) {
                return msg;
            }
            continue;
        }
        return SAAJFactory.instance.readAsSOAPMessage(soapVersion, message);
    }
    
    public static SOAPMessage read(SOAPVersion soapVersion, Message message, Packet packet) throws SOAPException {
        for (SAAJFactory s : ServiceFinder.find(SAAJFactory.class)) {
            SOAPMessage msg = s.readAsSOAPMessage(soapVersion, message, packet);
            if (msg != null) {
                return msg;
            }
            continue;
        }
        return SAAJFactory.instance.readAsSOAPMessage(soapVersion, message, packet);
    }
    
    public static SAAJMessage read(Packet packet) throws SOAPException {
        ServiceFinder<SAAJFactory> factories = (packet.component != null) ? ServiceFinder.find(SAAJFactory.class, packet.component) : ServiceFinder.find(SAAJFactory.class);
        for (SAAJFactory s : factories) {
            SAAJMessage msg = s.readAsSAAJ(packet);
            if (msg != null) {
                return msg;
            }
            continue;
        }
        return SAAJFactory.instance.readAsSAAJ(packet);
    }
    
    public SAAJMessage readAsSAAJ(Packet packet) throws SOAPException {
        SOAPVersion v = packet.getMessage().getSOAPVersion();
        SOAPMessage msg = this.readAsSOAPMessage(v, packet.getMessage());
        return new SAAJMessage(msg);
    }
    
    public MessageFactory createMessageFactory(String protocol) throws SOAPException {
        return MessageFactory.newInstance(protocol);
    }
    
    public SOAPFactory createSOAPFactory(String protocol) throws SOAPException {
        return SOAPFactory.newInstance(protocol);
    }
    
    public Message createMessage(SOAPMessage saaj) {
        return new SAAJMessage(saaj);
    }
    
    public SOAPMessage readAsSOAPMessage(SOAPVersion soapVersion, Message message) throws SOAPException {
        SOAPMessage msg = soapVersion.getMessageFactory().createMessage();
        SaajStaxWriter writer = new SaajStaxWriter(msg);
        try {
            message.writeTo(writer);
        }
        catch (XMLStreamException e) {
            throw (e.getCause() instanceof SOAPException) ? ((SOAPException)e.getCause()) : new SOAPException(e);
        }
        msg = writer.getSOAPMessage();
        addAttachmentsToSOAPMessage(msg, message);
        if (msg.saveRequired()) {
            msg.saveChanges();
            return msg;
        }
        return msg;
    }
    
    public SOAPMessage readAsSOAPMessageSax2Dom(SOAPVersion soapVersion, Message message) throws SOAPException {
        SOAPMessage msg = soapVersion.getMessageFactory().createMessage();
        SAX2DOMEx s2d = new SAX2DOMEx(msg.getSOAPPart());
        try {
            message.writeTo(s2d, XmlUtil.DRACONIAN_ERROR_HANDLER);
        }
        catch (SAXException e) {
            throw new SOAPException(e);
        }
        addAttachmentsToSOAPMessage(msg, message);
        if (msg.saveRequired()) {
            msg.saveChanges();
            return msg;
        }
        return msg;
    }
    
    protected static void addAttachmentsToSOAPMessage(SOAPMessage msg, Message message) {
        for (Attachment att : message.getAttachments()) {
            AttachmentPart part = msg.createAttachmentPart();
            part.setDataHandler(att.asDataHandler());
            String cid = att.getContentId();
            if (cid != null) {
                if (cid.startsWith("<") && cid.endsWith(">")) {
                    part.setContentId(cid);
                }
                else {
                    part.setContentId(60 + cid + 62);
                }
            }
            if ((att instanceof AttachmentEx)) {
                AttachmentEx ax = (AttachmentEx)att;
                Iterator<AttachmentEx.MimeHeader> imh = ax.getMimeHeaders();
                while (imh.hasNext()) {
                    AttachmentEx.MimeHeader ame = (AttachmentEx.MimeHeader)imh.next();
                    if (!"Content-ID".equals(ame.getName()) && !"Content-Type".equals(ame.getName())) {
                        part.addMimeHeader(ame.getName(), ame.getValue());
                    }
                    else {
                        continue;
                    }
                }
            }
            msg.addAttachmentPart(part);
        }
    }
    
    public SOAPMessage readAsSOAPMessage(SOAPVersion soapVersion, Message message, Packet packet) throws SOAPException {
        return this.readAsSOAPMessage(soapVersion, message);
    }
    
    static {
        instance = new SAAJFactory();
    }
}
